﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using InteractingWithDatabases;
using System.Data.SqlClient;
using System.Data;

namespace InteractingWithDatabases.Tests
{
    [TestFixture]
    public class DataAccessTests
    {
        DataAccess da = null;

        [SetUp]
        public void TestSetup()
        {
            da = new DataAccess();
        }

        [TearDown]
        public void TearDown()
        {
            da = null;
        }

        [Test]
        [TestCategory("Database")]
        public void GetConnStringFromAppConfig()
        {
            string actualString = da.ConnectionString;
            string expectedString = System.Configuration.ConfigurationManager.ConnectionStrings["DatabaseConnection"].ConnectionString;
            Assert.AreEqual(expectedString, actualString);
        }

        [Test]
        [TestCategory("Database")]
        public void ConnectAndDisconnectFromDatabase()
        {
            bool connected = da.Connect();

            bool disconnected = da.Disconnect();

            Assert.IsTrue(connected);
        }

        [Test]
        [TestCategory("Database")]
        public void GetSqlCommand()
        {
            string spoName = "spoTest";
            SqlCommand response = da.GetSqlCommand(spoName);

            Assert.IsNotNull(response);
            Assert.AreEqual(spoName, response.CommandText);
        }

        [Test]
        [TestCategory("Database")]
        public void ExecuteSqlCommand()
        {
            string spoName = "sp_who2";
            da.Connect();
            SqlCommand response = da.GetSqlCommand(spoName);
            DataTable ds = da.ExecuteSqlCommand(response);
            da.Disconnect();
            Assert.IsTrue(ds.Rows.Count > 0);
        }

        [Test]
        [TestCategory("Database")]
        [RollBack]
        public void InsertCustomerIntoDatabase()
        {
            string name = "Customer Test";
            string email = "Customer@Test.com";

            da.Connect();
            bool inserted = da.InsertCustomer(new Customer(name, email));
            da.Disconnect();
            Assert.IsTrue(inserted);
        }

        [Test]
        [TestCategory("Database")]
        [RollBack]
        public void UpdateCustomer()
        {
            string name = "Customer Test";
            string updatedName = "Updated Customer";
            da.Connect();

            da.InsertCustomer(new Customer(name, "Customer@Test.com"));

            Customer c = da.GetCustomer(name);

            c.Name = updatedName;
            da.UpdateCustomer(c);

            Customer c2 = da.GetCustomer(updatedName);

            da.Disconnect();

            Assert.AreEqual(updatedName, c2.Name);
            Assert.AreEqual(c.Email, c2.Email);
        }

        [Test]
        [TestCategory("Database")]
        [RollBack]
        public void GetCustomer()
        {
            string name = "Customer Test";
            string email = "Customer@Test.com";
            da.Connect();

            da.InsertCustomer(new Customer(name, email));

            Customer c = da.GetCustomer(name);

            da.Disconnect();

            Assert.IsNotNull(c);
            Assert.AreEqual(name, c.Name);
            StringAssert.IsNonEmpty(c.Email);
        }

        [Test]
        [TestCategory("Database")]
        [RollBack]
        public void DeleteCustomer()
        {
            string name = "Customer Test";
            string email = "Customer@Test.com";
            da.Connect();

            da.InsertCustomer(new Customer(name, email));

            Customer c = da.GetCustomer(name);

            da.DeleteCustomer(c);

            Customer c2 = da.GetCustomer(name);

            da.Disconnect();

            Assert.IsNull(c2);
        }

        [Test]
        [TestCategory("Database")]
        public void IfCustomerNotFoundReturnNull()
        {
            da.Connect();
            Customer c = da.GetCustomer("Unknown");
            da.Disconnect();
            Assert.IsNull(c);
        }

        [Test]
        [TestCategory("Database")]
        [RollBack]
        public void GetAllCustomers()
        {
            string name = "Customer Test";
            string email = "Customer@Test.com";
            int insertCount = 5;

            da.Connect();

            for (int i = 0; i < insertCount; i++)
            {
                da.InsertCustomer(new Customer(name + i, email));
                Console.WriteLine(name + i);

            }

            List<Customer> customers = da.GetAllCustomers();

            da.Disconnect();

            Assert.IsNotNull(customers);
            Assert.IsTrue(customers.Count == insertCount);
        }

        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void IfCustomerNameNullThrowArgumentException()
        {
            string name = null;
            string email = "Customer@Test.com";

            Customer c = new Customer(name, email);

            Assert.Fail("If we get here, an exception hasn't been thrown");
        }
    }
}
